/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.core;

// API classes
import com.rempl.api.Logger;
import com.rempl.api.RuntimeProblem;

// ANTLR3-generated parsers and lexers
import com.rempl.core.model.QueryLexer;
import com.rempl.core.model.QueryParser;

// ANTLR3 runtime for grammar/lexical processing
import org.antlr.runtime.ANTLRStringStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.TokenStream;

/**
 * Analyzer and parser of OCL query.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: QueryAnalyzer.java 1306 2011-05-02 16:04:25Z guard $
 * @see SimpleModel#query(String)
 */
final class QueryAnalyzer {

    /**
     * Analyzes the OCL query and returns the context name.
     * @param query OCL query
     * @return The OCL context found in the query
     * @todo #2 This implementation is very preliminary and
     *          should be replaced with a proper OCL parsing
     *          strategy.
     * @see SimpleModel#query(String)
     */
    public String analyze(final String query) {
        final CharStream input = new ANTLRStringStream(query);
        final QueryLexer lexer = new QueryLexer(input);
        final TokenStream tokens = new CommonTokenStream(lexer);
        final QueryParser parser = new QueryParser(tokens);
        String context;
        try {
            context = parser.context();
        } catch (org.antlr.runtime.RecognitionException ex) {
            throw RuntimeProblem.make(ex);
        }
        Logger.debug(
            this,
            "#query('%s'): found context '%s'",
            query,
            context
        );
        return context;
    }

}
